#import "FeedMe.h"
#import <libc.h>
#import <math.h>
#import <appkit/Application.h>
#import <objc/List.h>

static void handleFeedMe(DPSTimedEntry teNumber, double now, id object)
{
	[object changePrices];
	}
	
@implementation FeedMe

+ (char *)eventName
{
	return "FeedMe";
	}
	
- init
{
	id tmp,sheetList;
	int x;
	
	[super init];
	
	numSheets = 0;
	maxSheets = 16;
	sheets = malloc(maxSheets * sizeof(id));
	ranges = malloc(maxSheets * sizeof(char *));
	
	// go through the open sheets (if any) and look for ones that have Feed defined
	sheetList = [NXApp getOpenSheets];
	
	for (x = 0; x < [sheetList count]; x++) {
		tmp = [sheetList objectAt:x];
		
		// update our access to the script
		[self addInScriptChanged:tmp];
		}
	
	// call us every 2 seconds to update the prices
	// this could be an "DPSAddPort" if the data feed is coming from a Mach port
	te = DPSAddTimedEntry(3, (DPSTimedEntryProc) handleFeedMe, self, 15);

	return self;
	}
	
- free
{
	int x;
	
	// remove the timed entry
	DPSRemoveTimedEntry(te);
	
	// free each of the range names
	for (x = 0; x < numSheets; x++) free(ranges[x]);
	
	// free the range and sheet arrays
	free(sheets);
	free(ranges);
			
	return [super free];
	}
	
- sheetDidOpen:sender
{
	[self addInScriptChanged:sender];
	return self;
	}
	
- sheetDidClose:sender
{
	int x;
	
	// find the sheet that's going away
	for (x = 0; x < numSheets; x++) if (sender == sheets[x]) {
		
		// free the range
		free(ranges[x]);
		
		// decrement the number of sheets
		numSheets--;
		
		// compress the list of sheets
		for (;x < numSheets; x++) {
			ranges[x] = ranges[x + 1];
			sheets[x] = sheets[x + 1];
			}
		}
		
	return self;
	}
	
- addInScriptChanged:sender
{
	char *s,*s1;
	int x;
	
	// remove the sheet from the list if it's on the list
	[self sheetDidClose:sender];
	
	// get our event header
	s = [sender getEventHeader:"FeedMe"];
		
	// if the header has text 
	if (strlen(s)) {
		// a space or control character marks the end of a label
		for (s1 = s; *s1 > ' ';s1++);
		*s1 = 0;
		
		if (strlen(s)) {
			// if we are at the max with the number of sheets, grow the buffer
			if (numSheets == maxSheets) {
				maxSheets *= 2;
				sheets = realloc(sheets,maxSheets * sizeof(id));
				ranges = realloc(ranges,maxSheets * sizeof(char *));
				}
				
			sheets[numSheets] = sender;
			ranges[numSheets] = NXCopyStringBuffer(s);			
			numSheets++;
			}
		}
	
	// release the buffer passed by getEventHeader	
	free(s);
	
	return self;
	}

#define NUM 5
static char *stocks[] = {"FOO","ABC","XYZ","IBM","ATT"};
static double prices[] = {20,30,40,50,75};
	
- changePrices
{
	MOLIValue values[NUM];
	int x;
	double d;
	
	// randomize prices
	for (x = 0; x < NUM; x++) {
		d = prices[x] * 0.1;
		d *= (double) ((double) random() / (double) 2147483647);
		d -= prices[x] * 0.05;
		prices[x] += d;
		if (prices[x] < 1) prices[x] = 1;
		}
	
	// build the value strings
	for (x = 0; x < NUM; x++) {
		values[x].type = numberMOLIValue;
		values[x].values.number = prices[x];
		}
	
	// place the values in each sheet's table
	for (x = 0; x < numSheets; x++) {
		[sheets[x] associateLabel:ranges[x] with:stocks and:values num:NUM offset:1
			orientation:verticalOrientation];
		[sheets[x] setChanged:YES];
		[sheets[x] doRecalc];
		}
		
	return self;
	}

@end
