/**************************************************************************
 *                                                                        *
 *                                                                        *
 *          This code is copyright (c) 1992                               *
 *                     Athena Design, Inc.                                *
 *                     and David Pollak                                   *
 *                                                                        *
 *                ALL RIGHTS RESERVED                                     *
 *                                                                        *
 *                                                                        *
 *                                                                        *
 *                                                                        *
 *                                                                        *
 **************************************************************************/

#ifndef LITE
//#import <appkit/appkit.h>
#import <mach/mach.h>
#import <mach/message.h>
#import <appkit/View.h>

enum {stringMOLIValue, numberMOLIValue, errorMOLIValue};
enum {horizontalOrientation, verticalOrientation};

typedef struct _MOLIValue {
	int type;
	short row,col;
	union {
		char *string;
		double number;
		int error;
		} values;
	} MOLIValue;

typedef struct _MOLIMessage {
	msg_header_t head;
	msg_type_t otherPortType;
	port_t otherPort;
	msg_type_t handleType;
	int handle;
	msg_type_t lenType;
	int theLen;
	msg_type_long_t dataType;
	void *theData;
	} MOLIMessage;
	
enum {MOLISuccess, MOLISendError, MOLIReceiveError, MOLILabelNotFoundError,
	MOLISheetClosedError, MOLINotImplError, MOLIGraphNotFoundError,
	MOLIOutOfRangeError,MOLIReportNotFound,MOLILastError};
	
enum {openSheetMesaDo, unlinkSheetMesaDo, recalcMesaDo, associateMesaDo,
	getLabelRangeMesaDo, getRangeMesaDo, putRangeMesaDo,
	getLabelsMesaDo, saveMesaDo, saveAsMesaDo, getRawInputMesaDo, setRawInputMesaDo,
	hideMesaDo, orderFrontMesaDo, newSheetMesaDo, displayMesaDo,
	getEPSForLabelMesaDo, getEPSForRangeMesaDo, getEPSForGraphMesaDo,
	printReportMesaDo,didRecalcMesaDo,didRedisplayMesaDo, setColorAtPointInRangeMesaDo,
	setColorAtPointInLabelMesaDo, setColorAtPointInGraphMesaDo, setFontAtPointInRangeMesaDo,
	setFontAtPointInLabelMesaDo, setFontAtPointInGraphMesaDo,
	setRangeToFontMesaDo, setRangeToColorMesaDo, setRangeToBkgColorMesaDo,
	setLabelToFontMesaDo, setLabelToColorMesaDo, setLabelToBkgColorMesaDo,
	setInputMesaDo, performQueryMesaDo, getInputMesaDo, getCellForPointInRangeMesaDo,
	getCellForPointInLabelMesaDo, getRectForCellInRangeMesaDo, getRectForCellInLabelMesaDo,
	getRangeForLabelMesaDo, putRangePFMesaDo,
	createGraphMesaDo,setRangeToBorderMesaDo, setRangeToBorderThicknessMesaDo,
	setRangeToBorderColorMesaDo, setRangeToDisplayMesaDo, setRangeToPrecisionMesaDo,
	setRangeToAlignmentMesaDo, setGraphToRectMesaDo, setGraphToTypeMesaDo,
	setGraphToOrientationMesaDo, setGraphToLegendMesaDo, getMesaVersionMesaDo,
	getReportsMesaDo, getQueriesMesaDo, setDirtyMesaDo, setCleanMesaDo, unlinkSheetAndCloseMesaDo,
	runScriptMesaDo, removeLabelMesaDo, miniturizeMesaDo, getScriptMesaDo,
	getEventHeaderForAddInMesaDo, getEventHeaderMesaDo, setEventHeaderToForAddInMesaDo,
	addLabelMesaDo,setNumberOfRowsMesaDo,setNumberOfColsMesaDo,setBkgClearMesaDo,
	setCellHeightMesaDo,setCellWidthMesaDo,setAutoRecalcMesaDo,setGridOnMesaDo,
	setBkgColorMesaDo, setGridColorMesaDo,sortRangeMesaDo};

enum {noMesaLegend,leftMesaLegend,rightMesaLegend,topMesaLegend,bottomMesaLegend};

enum {generalMesaFormat = 0,fixedDecimalMesaFormat,scientificMesaFormat,currencyMesaFormat,
				commaMesaFormat,chartMesaFormat,percentMesaFormat,textMesaFormat,hiddenMesaFormat,
				dayMonthYearMesaFormat,dayMonthMesaFormat,monthYearMesaFormat,
				dateHourMinSecMesaFormat,
				dateHourMinMesaFormat,dateIntl1MesaFormat,dateIntl2MesaFormat,
				timeIntl1MesaFormat,timeIntl2MesaFormat};
enum {noMesaBorder = 0,thinMesaBorder,thickMesaBorder};
enum {leftMesaAlign = 0,rightMesaAlign,centerMesaAlign,smartMesaAlign,fillMesaAlign};

enum {scatterMesaGraphType = 2,columnMesaGraphType,
	barMesaGraphType,pieMesaGraphType,sColumnMesaGraphType,sBarMesaGraphType,
	areaMesaGraphType,HLCMesaGraphType,HLMesaGraphType,XYMesaGraphType,d3BarMesaGraphType,
	ribbonMesaGraphType,pyramidMesaGraphType,d3AreaMesaGraphType};

enum {anyMesaMode = 0,formulaMesaMode,numberMesaMode,stringMesaMode,dateMesaMode,
	unprotectedMesaMode};

void freeMOLIValue(int,MOLIValue *);

#define CHANGETOPBORDER 1
#define SETTOPBORDER 2
#define CHANGEBOTTOMBORDER 4
#define SETBOTTOMBORDER 8
#define CHANGELEFTBORDER 16
#define SETLEFTBORDER 32
#define CHANGERIGHTBORDER 64
#define SETRIGHTBORDER 128


@interface MesaObject : Object
{
	port_t myPort;
	port_t mesaPort;
	port_t localPort;
	MOLIMessage theMessage;
	id delegate;
}

- initToWorksheet:(const char *)ws;
- initToNewWorksheet;
- free;
- freeAndClose;
- (int)recalc;
- (int)displaySheet;
- (int)saveSheet;
- (int)saveSheetAs:(const char *)name;
- (int)hide;
- (int)makeKeyAndOrderFront;

- (int)getValues:(MOLIValue **)mv num:(int *)n
	upperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc;
- (int)putValues:(MOLIValue *)mv num:(int)n
	upperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc;
- (int)getLabels:(char ***)lp ranges:(short **)r num:(int *)num;
- (int)getRangeUpperRow:(int *)ur col:(int *)uc lowerRow:(int *)lr col:(int *)lc
	forLabel:(char *)label;

- (int)associateLabel:(const char *)l withItems:(char **)it andValues:(MOLIValue *)mv
	offset:(int)off numItems:(int)ni orientation:(int)or;
- (int)getValues:(MOLIValue **)mv  forLabel:(const char *)l num:(int *)n
	upperRow:(int *)ur col:(int *)uc lowerRow:(int *)lr col:(int *)lc;
- (int)getEPSForUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lowc into:(id *)vp;
- (int)getEPSForLabel:(char *)l in:(id *)vp;
- (int)getEPSForGraph:(char *)l in:(id *)vp;
- (int)printReport:(char *)r;
- (int)performSQLQuery:(char *)r;
- sheetDidRecalc:sender;
- sheetDidRedisplay:sender;
- (int)setColor:(NXColor)col atPoint:(NXPoint *)np inRangeUpperRow:(int)ur col:(int)uc
	lowerRow:(int)lr col:(int)lc flags:(int)fl;
- (int)setColor:(NXColor)col atPoint:(NXPoint *)np inLabel:(char *)l flags:(int)fl;
- (int)setColor:(NXColor)col atPoint:(NXPoint *)np inGraph:(char *)l flags:(int)fl;

- (int)setFont:font atPoint:(NXPoint *)np inRangeUpperRow:(int)ur col:(int)uc
	lowerRow:(int)lr col:(int)lc;
- (int)setFont:font atPoint:(NXPoint *)np inLabel:(char *)l;
- (int)setFont:font atPoint:(NXPoint *)np inGraph:(char *)l;

- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toFont:font;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toColor:(NXColor)col;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toBkgColor:(NXColor)col;
- (int)setLabel:(char *)l toFont:font;
- (int)setLabel:(char *)l toColor:(NXColor)col;
- (int)setLabel:(char *)l toBkgColor:(NXColor)col;

- (int)getInput:(char **)i forCell:(int)r :(int)c;
- (int)setInput:(char *)i forCell:(int)r :(int)c;

- (int)getCell:(int *)row :(int *)col forPoint:(NXPoint *)np inRangeUpperRow:(int)ur
	col:(int)uc lowerRow:(int)lr col:(int)lc;
- (int)getCell:(int *)row :(int *)col forPoint:(NXPoint *)np inLabel:(char *)l;

- (int)getRect:(NXRect *)re forCell:(int)r :(int)c inRangeUpperRow:(int)ur col:(int)uc
	lowerRow:(int)lr col:(int)lc;
- (int)getRect:(NXRect *)re forCell:(int)r :(int)c inLabel:(char *)l;

- setDelegate:del;
- delegate;

// stuff for v 1.2
- (int)putValuesAndPreserveFormatting:(MOLIValue *)mv num:(int)n
	upperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc;
- (int)createGraph:(int)type at:(NXRect *)re upperRow:(int)ur col:(int)uc
	lowerRow:(int)lr col:(int)lc name:(char **)gn;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toBorder:(int)border;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toBorderThickness:(int)bt;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc
	toBorderColor:(NXColor)bc;
- (int)setGraph:(char *)gn toRect:(NXRect *)re;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toDisplay:(int)bt;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toPrecision:(int)bt;
- (int)setRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc toAlignment:(int)bt;
- (int)setGraph:(char *)gn toType:(int)gt;
- (int)setGraph:(char *)gn toOrientation:(int)go;
- (int)setGraph:(char *)gn toLegend:(int)gl;

// stuff for 1.2 rev 106
- (int)mesaVersion:(int *)v;
- (int)getReportNames:(char **)names num:(int *)n;
- (int)getQueryNames:(char **)names num:(int *)n;

// Mesa 1.4 rev 140+
- (int)setSheetChanged:(BOOL)ch;

// Mesa 1.4 rev 145+
- (int)addLabel:(char *)s forRangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc;
- (int)removeLabel:(char *)s;
+ (const char *)getErrorText:(int)error;
- (int)miniturize:(int)win;
- (int)getScriptNames:(char **)names num:(int *)n;
- (int)runMScript:(char *)s;
- (int)getEventHeader:(char *)name into:(char **)header;
- (int)getEventHeaders:(char **)headers;
- (int)setEventHeaderTo:(char *)en forAddIn:(char *)ai;
- (int)setNumberOfRows:(int)nr;
- (int)setNumberOfCols:(int)nc;
- (int)setBkgColor:(NXColor)col;
- (int)setBkgClear:(int)cl;
- (int)setCellHeight:(int)ch;
- (int)setCellWidth:(int)cw;
- (int)setAutoRecalc:(int)ar;
- (int)setGridOn:(int)go;
- (int)setGridColor:(NXColor)nc;
- (int)sortARangeUpperRow:(int)ur col:(int)uc lowerRow:(int)lr col:(int)lc by:(int)rows
					key1:(int)k1 :(int)d1
					key2:(int)k2 :(int)d2
					key3:(int)k3 :(int)d3
					hasTitles:(int)hasTitles;

// these methods are private.  Do not use them
- (int)sendMessage:(int)m :(void *)data :(int)len;
- (int)receiveMessage;

@end

@interface MesaListen : Object
{
	port_t myPort;
	char *portName;
}

- initToPort:(char *)s;
- free;
- gotMessage:(MOLIValue *)mv num:(int)num forUpperRow:(int)ur upperCol:(int)uc lowerRow:(int)lr
	lowerCol:(int)lc;
@end

@interface MesaView : View
{
	id moli;
	id image;
	int ur,uc,lr,lc;
	char *label;
	char *graph;
	id textField;
	int curRow,curCol;
	int canEdit;
}

- initFrame:(NXRect *)re toSheet:(char *)sheet upperRow:(int)ur1 col:(int)uc1
	lowerRow:(int)lr1 col:(int)lc1;
- initFrame:(NXRect *)re toSheet:(char *)sheet toLabel:(char *)l;
- initFrame:(NXRect *)re toSheet:(char *)sheet toGraph:(char *)g;
- acceptColor:(NXColor)nc atPoint:(NXPoint *)np;
- acceptFont:font atPoint:(NXPoint *)np;
- free;
- getImage;
- drawSelf:(NXRect *)re :(int)n;
- sheetDidRedisplay:sender;
- mouseDown:(NXEvent *)ev;
- textDidEnd:sender endChar:(unsigned short)whyEnd;
- editCell:(int)row :(int)col;
- (BOOL)canEdit;
- setCanEdit:(BOOL)ce;
@end
#endif

