#import "MesaAddIn.h"
#import <objc/List.h>
#import <dpsclient/dpsclient.h>

// this is an addin function that collects a series of sheets that refer to it
// and automatically recalcs all those sheets every 2 seconds

@interface Recalc : MesaAddIn
{
	id sheets;
	DPSTimedEntry te;
}

+ (char *)funcName; 	// so that the function addin will recognize this addin
+ (char *)eventName;	// so that the event addin will recognize this addin
- execute:(void *)stack numberOfParams:(int)num sheet:(void *)sh;
- sheetDidClose:sender;
- pingMe;
- init;
- free;
@end

static void handlePingMe(DPSTimedEntry teNumber, double now, id object)
{
	[object pingMe];
	}
	

@implementation Recalc
+ (char *)funcName
{
	return "RECALC";
	}
	
+ (char *)eventName
{
	return "RECALC";
	}
	
- execute:(void *)stack numberOfParams:(int)num sheet:(void *)sh
{
	id tmpId;
	
	// get the id of the sheet that invoked this function
	tmpId = getSheetFromPointer(sh);
	
	// add it to the list of sheets we should ping...
	[sheets addObjectIfAbsent:tmpId];
	
	// if they tried to pass us more than 1 parameter, push an error
	if (num != 1) pushErrorOnStack(stack,badFormulaError);
	// otherwise leave the value on the stack.
	
	return self;
	}
	
- sheetDidClose:sender
{
	// remove a sheet that has subscribed when its closed
	// so that it's not recalced again (this would cause a nasty bomb)...
	[sheets removeObject:sender];
	return self;
	}
	
- pingMe
{
	// recalc the subscribed sheets
	[sheets makeObjectsPerform:@selector(doRecalc)];
	return self;
	}
	
- init
{
	[super init];

	// recalc the sheets every 2 seconds
	te = DPSAddTimedEntry(2, (DPSTimedEntryProc) handlePingMe, self, 15);

	sheets = [[List alloc] init];
	return self;
	}
	
- free
{
	// free the sheet list
	[sheets free];
	
	// and don't recalc anymore
	DPSRemoveTimedEntry(te);
	
	return [super free];
	}

@end
